#!/usr/bin/env python3

import ldap3
import sys

class LdapBase:
    url = 'ldap-teste.inf.ufsm.br'
    usuarios = 'ou=People,dc=inf,dc=ufsm,dc=br'
    grupos = 'ou=Group,dc=inf,dc=ufsm,dc=br'    
    
    def __init__(self):
        self.server = ldap3.Server(LdapBase.url, get_info=ldap3.ALL)

    def buscaLogin(self, login, campos=ldap3.ALL_ATTRIBUTES):
        res = self.conn.search( LdapBase.usuarios,
                          '(uid={0})'.format(login),
                          attributes=campos )
        if res: return self.conn.entries[0]
        else: return None

    def listaUsuarios(self, campos=ldap3.ALL_ATTRIBUTES):
        res = self.conn.search( LdapBase.usuarios,
                                '(uid=*)',
                                attributes=campos )
        if res: return self.conn.entries
        else: return None

    def buscaGrupo(self, grupo, campos=ldap3.ALL_ATTRIBUTES):
        res = self.conn.search( LdapBase.grupos,
                                '(cn={0})'.format(grupo),
                                attributes=campos )
        if res: return self.conn.entries[0]
        else: return None

    def listaGrupos(self, campos: object = ldap3.ALL_ATTRIBUTES) -> object:
        res = self.conn.search( LdapBase.grupos,
                                '(cn=*)',
                                attributes=campos )
        if res: return self.conn.entries
        else: return None

    # TODO
    def buscaNome(self, nome): pass

    def buscaGidGrupo(self, grupo):
        res = self.conn.search( LdapBase.grupos,
                                '(cn={0})'.format(grupo),
                                attributes=[ 'gidNumber' ] )
        if not res:
            return -1
        return self.conn.entries[0]['gidNumber']
        
    def buscaNovoUid(self):
        res = self.conn.search( LdapBase.usuarios,
                                '(uid=*)',
                                attributes=[ 'uidNumber' ] )
        if not res:
            return -1
        entradas = [ a.uidNumber.value for a in self.conn.entries ]
        entradas.sort()
        return ( entradas[-1] + 1 )
    
    def buscaNovoGid(self):
        res = self.conn.search( LdapBase.grupos,
                                '(gidNumber=*)',
                                attributes=[ 'gidNumber' ] )
        if not res:
            return -1
        entradas = [ a.gidNumber.value for a in self.conn.entries ]
        entradas.sort()        
        return ( entradas[-1] + 1 )

class Ldap(LdapBase):
    def __init__(self):
        LdapBase.__init__(self)
        self.conn = ldap3.Connection(self.server, auto_bind=True)

    def fecha(self):
        self.conn.unbind()
        
class LdapAdmin(LdapBase):
    admin = 'cn=admin,dc=inf,dc=ufsm,dc=br'
    senha = 'amigao'

    def __init__(self):
        LdapBase.__init__(self)
        self.conn = ldap3.Connection(self.server, LdapAdmin.admin,
                                     LdapAdmin.senha,
                                     auto_bind=True)

    def adicionaUsuario(self,
                        login, nome,
                        grupo='outros', aluno=True,
                        matricula=0, email=None, senha=None,
                        curso='CC'):
        # quebra nome por espaços
        nomes = nome.split()
        # entrada LDAP do usuário
        alvo = "uid={0},{1}".format(login, LdapBase.usuarios)
        # atributos do usuário
        atributos = {}
        atributos['uid'] = login
        atributos['cn'] = nome
        atributos['loginShell'] = '/bin/bash'
        atributos['uidNumber'] = self.buscaNovoUid()
        atributos['gidNumber'] = self.buscaGidGrupo(grupo).value
        if aluno:
            atributos['homeDirectory'] = '/home/alunos/{0}'.format(login)
        else:
            atributos['homeDirectory'] = '/home/profs/{0}'.format(login)
        atributos['gecos'] = nome
        atributos['objectClass'] = ['top', 'posixAccount', 'shadowAccount',
                                    'inetOrgPerson']
        atributos['shadowLastChange'] = -1
        atributos['shadowMin'] = 0
        atributos['shadowMax'] = 99999
        atributos['shadowInactive'] = -1
        atributos['shadowWarning'] = 7
        atributos['shadowExpire'] = 99999
        atributos['shadowFlag'] = -1
        atributos['description'] = -1
        # novos campos
        if email == None:
            atributos['mail'] = '{0}.inf.ufsm.br'.format(login)
        else:
            atributos['mail'] = email
        atributos['displayName'] = nomes[0]
        atributos['employeeNumber'] = matricula
        atributos['employeeType'] = 'Pessoa'
        atributos['o'] = 'UFSM'  # organizationName
        atributos['ou'] = curso  # organizationalUnitName
        atributos['l'] = 'Santa Maria, RS, BR'  # localityName
        atributos['sn'] = nomes[-1]  # surname
        atributos['initials'] = ''.join([c[0].upper() for c in nomes])
        if senha != None:
            hashed_pass = ldap3.utils.hashed.hashed(
                ldap3.HASHED_SALTED_SHA, senha)
            atributos['userPassword'] = hashed_pass
        res = self.conn.add(alvo, attributes=atributos)
        return res

    def adicionaGrupo(self, grupo):
        alvo = "cn={0},{1}".format(grupo, LdapBase.grupos)        
        atributos = {}
        atributos['cn'] = grupo
        atributos['gidNumber'] = self.buscaNovoGid()
        atributos['objectClass'] = ['top', 'posixGroup']
        atributos['userPassword'] = '{cript}'
        #print(str(atributos))
        res = self.conn.add(alvo, attributes=atributos)
        return res

    def mudaSenha(self, login, senha):
        alvo = "uid={0},{1}".format(login, LdapBase.usuarios)        
        hashed_pass = ldap3.utils.hashed.hashed(
            ldap3.HASHED_SALTED_SHA, senha)
        atributos = {}
        atributos['userPassword'] = [(ldap3.MODIFY_REPLACE, [hashed_pass])]
        #print(str(atributos))
        res = self.conn.modify( alvo, atributos )
        return res

    def mudaGrupo(self, login, grupo):
        alvo = "uid={0},{1}".format(login, LdapBase.usuarios)
        atributos = {}
        atributos['gidNumber'] = [(ldap3.MODIFY_REPLACE, self.buscaGidGrupo(grupo).value)]
        #print(str(atributos))
        res = self.conn.modify(alvo, atributos )
        return res

    def removeGrupo(self, grupo):
        alvo = "cn={0},{1}".format(grupo, LdapBase.grupos)
        res = self.conn.delete(alvo)
        return res

    def adicionaMembro(self, login, grupo):
        alvo = "cn={0},{1}".format(grupo, LdapBase.grupos)
        atributos = {}
        atributos['memberUid'] = [(ldap3.MODIFY_ADD, login)]
        res = self.conn.modify(alvo, atributos)
        return res

    def removeMembro(self, login, grupo):
        alvo = "cn={0},{1}".format(grupo, LdapBase.grupos)
        atributos = {}
        atributos['memberUid'] = [(ldap3.MODIFY_DELETE, login)]
        res = self.conn.modify(alvo, atributos)
        return res

    def mudaNome(self, login, nome):
        alvo = "uid={0},{1}".format(login, LdapBase.usuarios)
        atributos = {}
        atributos['cn'] = [(ldap3.MODIFY_REPLACE, nome)]
        atributos['gecos'] = [(ldap3.MODIFY_REPLACE, nome)]
        res = self.conn.modify(alvo, atributos)
        return res

    def mudaEmail(self, login, email):
        alvo = "uid={0},{1}".format(login, LdapBase.usuarios)
        atributos = {}
        atributos['mail'] = [(ldap3.MODIFY_REPLACE, email)]
        res = self.conn.modify(alvo, atributos)
        return res

    def mudaHome(self, login, home): pass

    

